{-------------------------------------------------------------------------------
    ** TStringManager 2.1 compatibility with TStringCollection version 1.6 **

New users can ignore/delete this file.

--------------------------------------------------------------------------------
TStringManager is a major update of TStringCollection.
Besides the name change, a lot of functions have been optimized, and the format
of some functions have changed.
This unit contains a TStringCollection class that "reimplements" some functions
from version 1.6 that have changed.

Last date - 22 Aug 2001
-------------------------------------------------------------------------------}

unit StrConMigrate;

interface
uses StrMan, SysUtils;

const
  gcIntCharSet      : TCharSet = ['0'..'9','-','+'];
  gcNumCharSet      : TCharSet = ['0'..'9',',','.','-','+'];
  gcFloatCharSet    : TCharSet = ['0'..'9',',','.','-','+','E','e'];
  gcAZCharSet       : TCharSet = ['A'..'Z','a'..'z'];


type
  TStringCollection = class(TStringManager)
   //Now called StripUp
   function StripUpcase(const s : string;c : char = ' ') : string;

   //Now called TrimUp
   function TrimUpcase(const s : string) : string;

   //Now called EnsureFirst
   function FirstIfNot(const SubStr : string;const s : string) : string;

   //Now called EnsureLast
   function LastIfNot(const SubStr : string;const s : string) : string;

   //Now called GetSingleSorted
   function GetSingle(const s1 : string; const s2 : String = '') : String;

   //New IsSame function ignores control characters
   function IsSame(const s1,s2 : string;StripChar : char = ' ') : boolean;
   function IsSameIC(const s1,s2 : string;StripChar : char = ' ') : boolean;
   function IsSameFirst(SubStr,s : string;StripChar : char = ' ') : boolean;
   function IsSameFirstIC(SubStr,s : string;StripChar : char = ' ') : boolean;

   //Params changed
   function Pos(const SubStr, s : string; StartPos : Cardinal; IgnoreCase : boolean = false) : integer;

   //Changed delims - now gcWordDelims by default
   function CountWords(const s : string;aDelims : TCharSet = [' ',#9,#10,#13]) : integer;
   function WordPos(SubStr : string; s : string; Delims : TCharSet = [' '];StartPos : Cardinal = 0) : integer;
   function HasWord(const word,s : string) : boolean;
   function HasWordIC(const word,s : string) : boolean;
   function FirstWord(const s: string;Delims : TCharSet = [' ']) : string;
   function LastWord(const s: string;Delims : TCharSet = [' ']) : string;

   //Now faster code and changed params
   function Replace(FindStr,ReplaceWith,InStr : string;
        WholeWord,CaseSensitive : boolean;Delims : TCharSet = [' ']) : string;  overload;
   function Replace(FindStr,ReplaceWith,InStr : string) : string; overload;
   function ReplaceFirst(FindStr,ReplaceAllStr,InStr : string; wholeword : boolean;
     CaseSensitive : boolean; Delims : TCharSet; var StartAt : integer) : string;
  end;

var sc : TStringCollection;
implementation




function TStringCollection.StripUpcase(const s : string;c : char = ' ') : string;
var
      i : Integer;
 aStart : Integer;
  aStop : Integer;
   RLen : Integer;
begin
  aStop := Length(s);
  //we first calculate the length without the outer spaces
  while (aStop>0) and (s[aStop]=c) do System.Dec(aStop);
  if aStop=0 then //only has spaces
  begin
    Result := '';
    exit;
  end;
  aStart := 1;
  while (aStart<=aStop) and (s[aStart]=c) do Inc(aStart);

  //Set length of result minus the outer spaces for fast result[i]:=
  SetLength(Result,(aStop-aStart)+1);
  RLen := 0;
  //Set relevant characters to result
  i := aStart;
  while i<=aStop do
  begin
    if (s[i]=c) then
    begin
      while (i+1<=aStop) and (s[i+1]=c) do inc(i);
    end;
    System.Inc(RLen);
    Result[RLen] := gaANSIUpperArray[s[i]];
    inc(i);
  end;
  SetLength(Result,RLen);  //set final length
end;

function TStringCollection.TrimUpcase(const s : string) : string;
var i, l: Integer;
begin
  l := Length(s);
  i := 1;
  while (i <= l) and (s[i] <= ' ') do Inc(i);
  if i > l then Result := '' else
  begin
    while s[l] <= ' ' do Dec(l);
    Result := Upcase(System.Copy(s, i, l - i + 1));
  end;
end;


function TStringCollection.IsSame(const s1,s2 : string;StripChar : char = ' ') : boolean;
begin
  if StripChar<>#0
  then Result := Strip(s1,StripChar)=Strip(s2,StripChar)
  else Result := s1=s2;
end;

function TStringCollection.IsSameIC(const s1,s2 : string;StripChar : char = ' ') : boolean;
begin
  if StripChar<>#0
  then Result := SysUtils.AnsiCompareText(Strip(s1,StripChar),Strip(s2,StripChar))=0
  else Result := SysUtils.AnsiCompareText(s1,s2)=0;
end;

function TStringCollection.IsSameFirst(SubStr,s : string;StripChar : char = ' ') : boolean;
begin
  if StripChar<>#0 then
  begin
    SubStr := Strip(SubStr,Stripchar);
    s := Strip(s,Stripchar);
  end;
  Result := SubStr=System.Copy(s,1,Length(SubStr));
end;

function TStringCollection.IsSameFirstIC(SubStr,s : string;StripChar : char = ' ') : boolean;
begin
  if StripChar<>#0 then
  begin
    SubStr := Strip(SubStr,Stripchar);
    s := Strip(s,Stripchar);
  end;
  Result := SysUtils.AnsiCompareText(SubStr,System.Copy(s,1,Length(SubStr)))=0;
end;

function TStringCollection.Pos(const SubStr, s : string; StartPos : Cardinal; IgnoreCase : boolean) : integer;
begin
  if IgnoreCase
    then result:=inherited Pos(SubStr,s,StartPos)
    else result:=inherited PosIC(SubStr,s,StartPos);
end;


function TStringCollection.CountWords(const s : string;aDelims : TCharSet = [' ',#9,#10,#13]) : integer;
var     i : integer;
   Status : word;
begin
  Result := 0;
  Status := 0;
  for i := 1 to length(s) do
  begin
    if s[i] in aDelims then
    begin
      if Status<>2 then inc(Result);
      Status := 2;
    end else Status:=1;
  end;
  if Status = 1 then inc(Result);
end;


function TStringCollection.FirstIfNot(const SubStr : string;const s : string) : string;
begin
  if IsFirst(SubStr,s)
   then result:=s
   else result:=SubStr+s;
end;


function TStringCollection.LastIfNot(const SubStr : string;const s : string) : string;
begin
  if IsLast(SubStr,s)
   then result:=s
   else result:=s+SubStr;
end;

{-------------------------------------------------------------------------------
 Example: GetSingle('hhwwwhhaw') > 'ahw'
 Example: GetSingle('hhwwwhhaw','123h') > '123ahw'
-------------------------------------------------------------------------------}
function TStringCollection.GetSingle(const s1 : string;const s2 : String = '') : String;
var i : Integer;
   cs : TCharSet;
begin
  Result := '';
  cs := [];
  for i := 1 to Length(s1) do cs := cs+[s1[i]];
  for i := 1 to Length(s2) do cs := cs+[s2[i]];
  Result := Str(cs);
end;


function TStringCollection.HasWord(const word,s : string) : boolean;
begin
  Result := WordPos(word,s)>0;
end;

function TStringCollection.HasWordIC(const word,s : string) : boolean;
begin
  Result := WordPos(UpCase(word),UpCase(s))>0;
end;


function TStringCollection.WordPos(SubStr : string; s : string; Delims : TCharSet = [' '];StartPos : Cardinal = 0) : integer;
var
  p,n : integer;
  ok  : boolean;
begin
  Result := 0;
  p := StartPos;
  repeat
    n := Pos(SubStr,s,p);
    if n=0 then exit;
    ok := true;
    if n>1 then
    begin
      if s[n-1] in Delims then else ok := false;
    end;
    if n+Length(SubStr)<=Length(s) then
    begin
      if s[n+Length(SubStr)] in Delims then else ok := false;
    end;
    if ok then
    begin
      Result := n;
      exit;
    end;
    p := p + Length(SubStr);
  until (p=0);
end;

function TStringCollection.FirstWord(const s: string;Delims : TCharSet = [' ']) : string;
begin
  Result := Before(Delims,s);
end;

function TStringCollection.LastWord(const s: string;Delims : TCharSet = [' ']) : string;
begin
  Result := AfterRev(Delims,s);
end;

function TStringCollection.Replace(FindStr,ReplaceWith,InStr : string;
        WholeWord,CaseSensitive : boolean;Delims : TCharSet = [' ']) : string;
begin
  Result := inherited Replace(FindStr,ReplaceWith,InStr,not CaseSensitive,WholeWord,false,Delims);
end;

function TStringCollection.Replace(FindStr,ReplaceWith,InStr : string) : string;
begin
  Result := inherited Replace(FindStr,ReplaceWith,InStr,false,false,false,[]);
end;


function TStringCollection.ReplaceFirst(FindStr,ReplaceAllStr,InStr : string;
                            Wholeword,CaseSensitive : boolean;
                            Delims : TCharSet;var StartAt : integer ) : string;
begin
  Result := inherited ReplaceOnce(FindStr,ReplaceAllStr,InStr,StartAt,not CaseSensitive,WholeWord,false,Delims);
end;


initialization
  sc := TStringCollection.Create;

finalization
  sc.Free;

end.

